@@ -7,13 +7,11 @@
 // notice may not be copied, modified, or distributed except
 // according to those terms.
 
-use std::cmp;
-
-use proc_macro2::{Ident, TokenStream};
-
-use crate::enums::{Decoration, Dim, ImageFormat, StorageClass};
 use crate::parse::{Instruction, Spirv};
 use crate::{spirv_search, TypesMeta};
+use proc_macro2::TokenStream;
+use spirv_headers::{Decoration, Dim, ImageFormat, StorageClass};
+use std::cmp;
 use std::collections::HashSet;
 
 #[derive(Debug)]
@@ -25,18 +23,69 @@ struct Descriptor {
     readonly: bool,
 }
 
-pub(super) fn write_descriptor_sets(
+pub(super) fn write_descriptor_set_layout_descs(
     doc: &Spirv,
-    entry_point_layout_name: &Ident,
     entrypoint_id: u32,
     interface: &[u32],
-    types_meta: &TypesMeta,
     exact_entrypoint_interface: bool,
+    stages: &TokenStream,
 ) -> TokenStream {
     // TODO: somewhat implemented correctly
 
     // Finding all the descriptors.
     let descriptors = find_descriptors(doc, entrypoint_id, interface, exact_entrypoint_interface);
+    let num_sets = descriptors.iter().map(|d| d.set + 1).max().unwrap_or(0);
+    let sets: Vec<_> = (0..num_sets)
+        .map(|set_num| {
+            let num_bindings = descriptors
+                .iter()
+                .filter(|d| d.set == set_num)
+                .map(|d| d.binding + 1)
+                .max()
+                .unwrap_or(0);
+            let bindings: Vec<_> = (0..num_bindings)
+                .map(|binding_num| {
+                    match descriptors
+                        .iter()
+                        .find(|d| d.set == set_num && d.binding == binding_num)
+                    {
+                        Some(d) => {
+                            let desc_ty = &d.desc_ty;
+                            let array_count = d.array_count as u32;
+                            let readonly = d.readonly;
+                            quote! {
+                                Some(DescriptorDesc {
+                                    ty: #desc_ty,
+                                    array_count: #array_count,
+                                    stages: #stages,
+                                    readonly: #readonly,
+                                }),
+                            }
+                        }
+                        None => quote! {
+                            None,
+                        },
+                    }
+                })
+                .collect();
+
+            quote! {
+                DescriptorSetDesc::new(
+                    [#( #bindings )*]
+                ),
+            }
+        })
+        .collect();
+
+    quote! {
+        [
+            #( #sets )*
+        ]
+    }
+}
+
+pub(super) fn write_push_constant_ranges(doc: &Spirv, stage: &TokenStream, types_meta: &TypesMeta) -> TokenStream {
+    // TODO: somewhat implemented correctly
 
     // Looping to find all the push constant structs.
     let mut push_constants_size = 0;
@@ -44,7 +93,7 @@ pub(super) fn write_descriptor_sets(
         let type_id = match instruction {
             &Instruction::TypePointer {
                 type_id,
-                storage_class: StorageClass::StorageClassPushConstant,
+                storage_class: StorageClass::PushConstant,
                 ..
             } => type_id,
             _ => continue,
@@ -55,86 +104,19 @@ pub(super) fn write_descriptor_sets(
         push_constants_size = cmp::max(push_constants_size, size);
     }
 
-    // Writing the body of the `descriptor` method.
-    let descriptor_body = descriptors
-        .iter()
-        .map(|d| {
-            let set = d.set as usize;
-            let binding = d.binding as usize;
-            let desc_ty = &d.desc_ty;
-            let array_count = d.array_count as u32;
-            let readonly = d.readonly;
-            quote! {
-                (#set, #binding) => Some(DescriptorDesc {
-                    ty: #desc_ty,
-                    array_count: #array_count,
-                    stages: self.0.clone(),
-                    readonly: #readonly,
-                }),
-            }
-        })
-        .collect::<Vec<_>>();
-
-    let num_sets = descriptors.iter().fold(0, |s, d| cmp::max(s, d.set + 1)) as usize;
-
-    // Writing the body of the `num_bindings_in_set` method.
-    let num_bindings_in_set_body = (0..num_sets)
-        .map(|set| {
-            let num = descriptors
-                .iter()
-                .filter(|d| d.set == set as u32)
-                .fold(0, |s, d| cmp::max(s, 1 + d.binding)) as usize;
-            quote! { #set => Some(#num), }
-        })
-        .collect::<Vec<_>>();
-
-    // Writing the body of the `num_push_constants_ranges` method.
-    let num_push_constants_ranges_body = if push_constants_size == 0 { 0 } else { 1 } as usize;
-
-    // Writing the body of the `push_constants_range` method.
-    let push_constants_range_body = quote!(
-        if num != 0 || #push_constants_size == 0 {
+    if push_constants_size == 0 {
+        quote! {
             None
-        } else {
-            Some(PipelineLayoutDescPcRange {
-                offset: 0,                   // FIXME: not necessarily true
-                size: #push_constants_size,
-                stages: ShaderStages::all(), // FIXME: wrong
-            })
         }
-    );
-
-    quote! {
-        #[derive(Debug, Clone)]
-        pub struct #entry_point_layout_name(pub ShaderStages);
-
-        #[allow(unsafe_code)]
-        unsafe impl PipelineLayoutDesc for #entry_point_layout_name {
-            fn num_sets(&self) -> usize {
-                #num_sets
-            }
-
-            fn num_bindings_in_set(&self, set: usize) -> Option<usize> {
-                match set {
-                    #( #num_bindings_in_set_body )*
-                    _ => None
+    } else {
+        quote! {
+            Some(
+                PipelineLayoutPcRange {
+                    offset: 0,                   // FIXME: not necessarily true
+                    size: #push_constants_size,
+                    stages: #stage,
                 }
-            }
-
-            fn descriptor(&self, set: usize, binding: usize) -> Option<DescriptorDesc> {
-                match (set, binding) {
-                    #( #descriptor_body )*
-                    _ => None
-                }
-            }
-
-            fn num_push_constants_ranges(&self) -> usize {
-                #num_push_constants_ranges_body
-            }
-
-            fn push_constants_range(&self, num: usize) -> Option<PipelineLayoutDescPcRange> {
-                #push_constants_range_body
-            }
+            )
         }
     }
 }
@@ -166,7 +148,7 @@ fn find_descriptors(
     };
 
     // Looping to find all the interface elements that have the `DescriptorSet` decoration.
-    for set_decoration in doc.get_decorations(Decoration::DecorationDescriptorSet) {
+    for set_decoration in doc.get_decorations(Decoration::DescriptorSet) {
         let variable_id = set_decoration.target_id;
 
         if exact && !variables.as_ref().unwrap().contains(&variable_id) {
@@ -183,11 +165,11 @@ fn find_descriptors(
         // Find the binding point of this descriptor.
         // TODO: There was a previous todo here, I think it was asking for this to be implemented for member decorations? check git history
         let binding = doc
-            .get_decoration_params(variable_id, Decoration::DecorationBinding)
+            .get_decoration_params(variable_id, Decoration::Binding)
             .unwrap()[0];
 
         let nonwritable = doc
-            .get_decoration_params(variable_id, Decoration::DecorationNonWritable)
+            .get_decoration_params(variable_id, Decoration::NonWritable)
             .is_some();
 
         // Find information about the kind of binding for this descriptor.
@@ -393,10 +375,10 @@ fn descriptor_infos(
             match i {
                 Instruction::TypeStruct { result_id, member_types } if *result_id == pointed_ty => {
                     let decoration_block = doc
-                        .get_decoration_params(pointed_ty, Decoration::DecorationBlock)
+                        .get_decoration_params(pointed_ty, Decoration::Block)
                         .is_some();
                     let decoration_buffer_block = doc
-                        .get_decoration_params(pointed_ty, Decoration::DecorationBufferBlock)
+                        .get_decoration_params(pointed_ty, Decoration::BufferBlock)
                         .is_some();
                     assert!(
                         decoration_block ^ decoration_buffer_block,
@@ -405,11 +387,11 @@ fn descriptor_infos(
 
                     // false -> VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER
                     // true -> VK_DESCRIPTOR_TYPE_STORAGE_BUFFER
-                    let storage = pointer_storage == StorageClass::StorageClassStorageBuffer;
+                    let storage = decoration_buffer_block || decoration_block && pointer_storage == StorageClass::StorageBuffer;
 
                     // Determine whether all members have a NonWritable decoration.
                     let nonwritable = (0..member_types.len() as u32).all(|i| {
-                        doc.get_member_decoration_params(pointed_ty, i, Decoration::DecorationNonWritable).is_some()
+                        doc.get_member_decoration_params(pointed_ty, i, Decoration::NonWritable).is_some()
                     });
 
                     // Uniforms are never writable.
@@ -438,6 +420,8 @@ fn descriptor_infos(
                                               have a Sampled operand of 1 or 2",
                     );
 
+                    let vulkan_format = to_vulkan_format(*format);
+
                     let arrayed = match arrayed {
                         true => quote! { DescriptorImageDescArray::Arrayed { max_layers: None } },
                         false => quote! { DescriptorImageDescArray::NonArrayed },
@@ -453,11 +437,7 @@ fn descriptor_infos(
                                                                 SubpassData"
                             );
                             assert!(
-                                if let &ImageFormat::ImageFormatUnknown = format {
-                                    true
-                                } else {
-                                    false
-                                },
+                                *format == ImageFormat::Unknown,
                                 "If Dim is SubpassData, Image Format must be Unknown"
                             );
                             assert!(!sampled, "If Dim is SubpassData, Sampled must be 2");
@@ -478,7 +458,7 @@ fn descriptor_infos(
                             let desc = quote! {
                                 DescriptorDescTy::TexelBuffer {
                                     storage: #storage,
-                                    format: None, // TODO: specify format if known
+                                    format: #vulkan_format,
                                 }
                             };
 
@@ -515,7 +495,7 @@ fn descriptor_infos(
                                 #ty(DescriptorImageDesc {
                                     sampled: #sampled,
                                     dimensions: #dim,
-                                    format: None,       // TODO: specify format if known
+                                    format: #vulkan_format,
                                     multisampled: #ms,
                                     array_layers: #arrayed,
                                 })
@@ -723,6 +703,8 @@ mod tests {
             ShaderKind::Vertex,
             &includes,
             &defines,
+            None,
+            None,
         )
         .unwrap();
         let doc = parse::parse_spirv(comp.as_binary()).unwrap();
@@ -749,3 +731,48 @@ mod tests {
         panic!("Could not find entrypoint");
     }
 }
+
+fn to_vulkan_format(spirv_format: ImageFormat) -> TokenStream {
+    match spirv_format {
+        ImageFormat::Unknown => quote! { None },
+        ImageFormat::Rgba32f => quote! { Some(Format::R32G32B32A32Sfloat) },
+        ImageFormat::Rgba16f => quote! { Some(Format::R16G16B16A16Sfloat) },
+        ImageFormat::R32f => quote! { Some(Format::R32Sfloat) },
+        ImageFormat::Rgba8 => quote! { Some(Format::R8G8B8A8Unorm) },
+        ImageFormat::Rgba8Snorm => quote! { Some(Format::R8G8B8A8Snorm) },
+        ImageFormat::Rg32f => quote! { Some(Format::R32G32Sfloat) },
+        ImageFormat::Rg16f => quote! { Some(Format::R16G16Sfloat) },
+        ImageFormat::R11fG11fB10f => quote! { Some(Format::B10G11R11UfloatPack32) },
+        ImageFormat::R16f => quote! { Some(Format::R16Sfloat) },
+        ImageFormat::Rgba16 => quote! { Some(Format::R16G16B16A16Unorm) },
+        ImageFormat::Rgb10A2 => quote! { Some(Format::A2B10G10R10UnormPack32) },
+        ImageFormat::Rg16 => quote! { Some(Format::R16G16Unorm) },
+        ImageFormat::Rg8 => quote! { Some(Format::R8G8Unorm) },
+        ImageFormat::R16 => quote! { Some(Format::R16Unorm) },
+        ImageFormat::R8 => quote! { Some(Format::R8Unorm) },
+        ImageFormat::Rgba16Snorm => quote! { Some(Format::R16G16B16A16Snorm) },
+        ImageFormat::Rg16Snorm => quote! { Some(Format::R16G16Snorm) },
+        ImageFormat::Rg8Snorm => quote! { Some(Format::R8G8Snorm) },
+        ImageFormat::R16Snorm => quote! { Some(Format::R16Snorm) },
+        ImageFormat::R8Snorm => quote! { Some(Format::R8Snorm) },
+        ImageFormat::Rgba32i => quote! { Some(Format::R32G32B32A32Sint) },
+        ImageFormat::Rgba16i => quote! { Some(Format::R16G16B16A16Sint) },
+        ImageFormat::Rgba8i => quote! { Some(Format::R8G8B8A8Sint) },
+        ImageFormat::R32i => quote! { Some(Format::R32Sint) },
+        ImageFormat::Rg32i => quote! { Some(Format::R32G32Sint) },
+        ImageFormat::Rg16i => quote! { Some(Format::R16G16Sint) },
+        ImageFormat::Rg8i => quote! { Some(Format::R8G8Sint) },
+        ImageFormat::R16i => quote! { Some(Format::R16Sint) },
+        ImageFormat::R8i => quote! { Some(Format::R8Sint) },
+        ImageFormat::Rgba32ui => quote! { Some(Format::R32G32B32A32Uint) },
+        ImageFormat::Rgba16ui => quote! { Some(Format::R16G16B16A16Uint) },
+        ImageFormat::Rgba8ui => quote! { Some(Format::R8G8B8A8Uint) },
+        ImageFormat::R32ui => quote! { Some(Format::R32Uint) },
+        ImageFormat::Rgb10a2ui => quote! { Some(Format::A2B10G10R10UintPack32) },
+        ImageFormat::Rg32ui => quote! { Some(Format::R32G32Uint) },
+        ImageFormat::Rg16ui => quote! { Some(Format::R16G16Uint) },
+        ImageFormat::Rg8ui => quote! { Some(Format::R8G8Uint) },
+        ImageFormat::R16ui => quote! { Some(Format::R16Uint) },
+        ImageFormat::R8ui => quote! { Some(Format::R8Uint) },
+    }
+}
